﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace ReportDesignerApp.Forms
{

    public partial class ParametersForm : Form
    {
        C1.C1Report.C1Report _report;
        string _sSQLQuery = string.Empty;
        string _sAddedSQLQuery = string.Empty;
        string _sAddedSQLFilter = string.Empty;
        string _sOrderByQuery = string.Empty;
        

        public ParametersForm(C1.C1Report.C1Report rpt)
        {
            InitializeComponent();
            _report = new C1.C1Report.C1Report();
            _report = rpt;
        }

        private void ParametersForm_Load(object sender, EventArgs e)
        {
            try
            {
                if (_report.DataSource.RecordSource.IndexOf("SELECT", StringComparison.OrdinalIgnoreCase) != -1)
                {
                    _sSQLQuery = _report.DataSource.RecordSource;
                }
                else
                {
                    _sSQLQuery = "SELECT * FROM " + _report.DataSource.RecordSource.ToString();
                    _report.DataSource.RecordSource = _sSQLQuery;
                }

                if (_sSQLQuery != null)
                {
                    if (_sSQLQuery.IndexOf("ORDER BY", StringComparison.OrdinalIgnoreCase) != -1)
                    {
                        _sOrderByQuery = After(_sSQLQuery, _sSQLQuery.Substring(_sSQLQuery.IndexOf("ORDER BY",StringComparison.OrdinalIgnoreCase),9));
                        if (_sOrderByQuery.EndsWith(";"))
                            _sOrderByQuery = _sOrderByQuery.Remove(_sOrderByQuery.Length - 1);
                    }

                    if (_sSQLQuery.IndexOf("PARAMETERS", StringComparison.OrdinalIgnoreCase) != -1)
                    {
                        string _sParamName, _sParamType, _sDefaultValue;
                        _sSQLQuery = _sSQLQuery.Remove(0, 11);
                        string[] pieces = _sSQLQuery.Split(new[] { ";" }, StringSplitOptions.None);
                        string[] existingParameters = pieces[0].Split(new[] { "," }, StringSplitOptions.None);
                        foreach (string parameter in existingParameters)
                        {
                            _sParamName = Between(parameter, "[", "]");
                            if (parameter.IndexOf("String", StringComparison.OrdinalIgnoreCase) != -1)
                                _sParamType = Between(parameter, "] ", " \"");
                            else
                                _sParamType = Between(parameter, "] ", " ");
                            if(_sParamType.IndexOf("String", StringComparison.OrdinalIgnoreCase) !=-1)
                                _sDefaultValue = Between(parameter, _sParamType + " \"", "\"");
                            else
                                _sDefaultValue = After(parameter, _sParamType + " ");
                            _lbAddedParams.Items.Add(_sParamName + "--" + _sParamType + "--" + _sDefaultValue);
                            _lbParamsList.Items.Add(_sParamName);
                        }                        
                        _sSQLQuery = pieces[1];                        
                    }

                    if (_sSQLQuery.IndexOf("WHERE", StringComparison.OrdinalIgnoreCase) != -1)
                    {
                        string _sFilterQuery;
                        string[] _filters;
                        if (_sSQLQuery.IndexOf("ORDER BY", StringComparison.OrdinalIgnoreCase) != -1)
                        {
                            _sFilterQuery = Between(_sSQLQuery, _sSQLQuery.Substring(_sSQLQuery.IndexOf("WHERE", StringComparison.OrdinalIgnoreCase), 5), _sSQLQuery.Substring(_sSQLQuery.IndexOf("ORDER BY", StringComparison.OrdinalIgnoreCase), 9));
                            if (_sFilterQuery.EndsWith(";") || _sFilterQuery.EndsWith(","))
                                _sFilterQuery.Remove(_sFilterQuery.Length - 1);
                        }
                        else
                        {
                            _sFilterQuery = After(_sSQLQuery, _sSQLQuery.Substring(_sSQLQuery.IndexOf("WHERE", StringComparison.OrdinalIgnoreCase), 5));
                            if (_sFilterQuery.EndsWith(";") || _sFilterQuery.EndsWith(","))
                                _sFilterQuery = _sFilterQuery.Remove(_sFilterQuery.Length - 1);
                        }
                        _filters = _sFilterQuery.Split(new[] { "AND" }, StringSplitOptions.None);
                        foreach (string filter in _filters)
                        {
                            _lbFiltersList.Items.Add(filter);
                        }
                        _sSQLQuery = _sSQLQuery.Remove(_sSQLQuery.LastIndexOf("WHERE ", StringComparison.OrdinalIgnoreCase));
                    }
                    if (_sSQLQuery.IndexOf("ORDER BY", StringComparison.OrdinalIgnoreCase) != -1)
                        _sSQLQuery = _sSQLQuery.Remove(_sSQLQuery.LastIndexOf("ORDER BY", StringComparison.OrdinalIgnoreCase));
                    
                    //Fields List
                    string[] dbFields = _report.DataSource.GetDBFieldList(true);
                    foreach (string fld in dbFields)
                    {
                        _lbFieldList.Items.Add(fld);
                    }
                }

                _cbDefaultValue.Items.Add("True");
                _cbDefaultValue.Items.Add("False");

                _cbParamType.Items.Add("DateTime");
                _cbParamType.Items.Add("Short");
                _cbParamType.Items.Add("String");
                _cbParamType.Items.Add("Boolean");

                _cbOperatorType.Items.Add("=");
                _cbOperatorType.Items.Add("<>");
                _cbOperatorType.Items.Add(">");
                _cbOperatorType.Items.Add("<");
                _cbOperatorType.Items.Add(">=");
                _cbOperatorType.Items.Add("<=");
                _cbOperatorType.Items.Add("BETWEEN");
                _cbOperatorType.DropDownWidth = 200;

                _cbDefaultValue.Visible = false;
                _dtpDefaultValue.Visible = false;
                _nudDefaultValue.Visible = false;
                _tbDefaultValue.Visible = false;

                _nudDefaultValue.Maximum = 1000000;
                _nudDefaultValue.Minimum = -1000000;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        private string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private void _btnAddNewParam_Click(object sender, EventArgs e)
        {
            _tbParamName.Clear();
            _cbParamType.SelectedIndex = -1;
            _tbDefaultValue.Clear();
            _tbDefaultValue.Visible = false;
            _cbDefaultValue.SelectedIndex = -1;
            _cbDefaultValue.Visible = false;
            _dtpDefaultValue.Value = DateTime.Today;
            _dtpDefaultValue.Visible = false;
            _nudDefaultValue.Value = 0;
            _nudDefaultValue.Visible = false;
        }

        private void _btnAddParam_Click(object sender, EventArgs e)
        {
            bool _bAlreadyExists = false;
            foreach (string item in _lbAddedParams.Items)
            {
                string[] delimiterChars = { "--" };
                string[] words = item.Split(delimiterChars, StringSplitOptions.None);
                if (words[0] == _tbParamName.Text)
                {
                    _bAlreadyExists = true;
                    MessageBox.Show("Duplicate Parameter Name");
                    _tbParamName.Text = string.Empty;
                }
            }
            if (!_bAlreadyExists)
            {
                try
                {
                    string _sDefaultValue = string.Empty;
                    if (_cbDefaultValue.Visible)
                        _sDefaultValue = _cbDefaultValue.SelectedItem.ToString();
                    else if (_tbDefaultValue.Visible)
                        _sDefaultValue = _tbDefaultValue.Text;
                    else if (_dtpDefaultValue.Visible)
                        _sDefaultValue = _dtpDefaultValue.Value.ToShortDateString();
                    else if (_nudDefaultValue.Visible)
                        _sDefaultValue = _nudDefaultValue.Value.ToString();
                    _lbAddedParams.Items.Add(_tbParamName.Text + "--" + _cbParamType.SelectedItem.ToString() + "--" + _sDefaultValue);
                    _lbParamsList.Items.Add(_tbParamName.Text);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void _btnRemoveAll_Click(object sender, EventArgs e)
        {
            _lbFiltersList.Items.Clear();
            _lbAddedParams.Items.Clear();
            _lbParamsList.Items.Clear();
        }

        private void _btnRemoveSelected_Click(object sender, EventArgs e)
        {
            try
            {
                if (_lbAddedParams.SelectedIndex != -1)
                {
                    string[] pieces = _lbAddedParams.SelectedItem.ToString().Split(new[] { "--" }, StringSplitOptions.None);
                    for (int i = 0; i < _lbFiltersList.Items.Count; i++)
                    {
                        if (_lbFiltersList.Items[i].ToString().IndexOf(pieces[0],StringComparison.OrdinalIgnoreCase)!= -1)
                        {
                            _lbFiltersList.Items.RemoveAt(i);
                        }
                    }
                    _lbParamsList.Items.Remove(pieces[0]);                    
                    _lbAddedParams.Items.RemoveAt(_lbAddedParams.SelectedIndex);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void _btnClear_Click(object sender, EventArgs e)
        {
            _tbParamName.Clear();
            _cbParamType.SelectedIndex = -1;
            _tbDefaultValue.Clear();
            _tbDefaultValue.Visible = false;
            _cbDefaultValue.SelectedIndex = -1;
            _cbDefaultValue.Visible = false;
            _dtpDefaultValue.Value = DateTime.Today;
            _dtpDefaultValue.Visible = false;
            _nudDefaultValue.Value = 0;
            _nudDefaultValue.Visible = false;
        }

        private void _cbParamType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_cbParamType.SelectedIndex != -1)
            {
                if (_cbParamType.SelectedItem.ToString() == "DateTime")
                {
                    _dtpDefaultValue.Visible = true;
                    _nudDefaultValue.Visible = false;
                    _tbDefaultValue.Visible = false;
                    _cbDefaultValue.Visible = false;
                }
                else if (_cbParamType.SelectedItem.ToString() == "Short")
                {
                    _nudDefaultValue.Visible = true;
                    _dtpDefaultValue.Visible = false;
                    _tbDefaultValue.Visible = false;
                    _cbDefaultValue.Visible = false;
                }
                else if (_cbParamType.SelectedItem.ToString() == "String")
                {
                    _tbDefaultValue.Visible = true;
                    _dtpDefaultValue.Visible = false;
                    _nudDefaultValue.Visible = false;
                    _cbDefaultValue.Visible = false;
                }
                else if (_cbParamType.SelectedItem.ToString() == "Boolean")
                {
                    _cbDefaultValue.Visible = true;
                    _dtpDefaultValue.Visible = false;
                    _nudDefaultValue.Visible = false;
                    _tbDefaultValue.Visible = false;
                }
            }
        }

        private void _btnOK_Click(object sender, EventArgs e)
        {
            try
            {
                string[] delimiterChars = { "--" };
                //if (_lbAddedParams.Items.Count > 0 || _lbFiltersList.Items.Count > 0)
                //{
                    foreach (string parameter in _lbAddedParams.Items)
                    {
                        string[] words = parameter.Split(delimiterChars, StringSplitOptions.None);
                        _sAddedSQLQuery = _sAddedSQLQuery + " [" + words[0] + "] " + words[1];
                        if (words[1] == "DateTime" || words[1] == "Boolean" || words[1] == "Short")
                            _sAddedSQLQuery = _sAddedSQLQuery + " " + words[2];
                        else if (words[1] == "String")
                        {
                            if (words[2].StartsWith("\""))
                                words[2] = words[2].Remove(0, 1);
                            if (words[2].EndsWith("\""))
                                words[2] = words[2].Remove(words[2].Length - 1, 1);

                            _sAddedSQLQuery = _sAddedSQLQuery + " \"" + words[2] + "\"";
                        }
                        _sAddedSQLQuery = _sAddedSQLQuery + ",";
                    }
                    if (_sAddedSQLQuery.EndsWith(",") || _sAddedSQLQuery.EndsWith(";"))
                        _sAddedSQLQuery = _sAddedSQLQuery.Remove(_sAddedSQLQuery.Length - 1);
                    if(_lbAddedParams.Items.Count>0)
                        _sAddedSQLQuery = _sAddedSQLQuery + ";";

                    foreach (string filter in _lbFiltersList.Items)
                    {
                        _sAddedSQLFilter = _sAddedSQLFilter + filter + " AND ";
                    }
                    if (_sAddedSQLFilter.EndsWith(",") || _sAddedSQLFilter.EndsWith(";"))
                        _sAddedSQLFilter = _sAddedSQLFilter.Remove(_sAddedSQLFilter.Length - 1);
                    else if (_sAddedSQLFilter.EndsWith(" AND "))
                        _sAddedSQLFilter = _sAddedSQLFilter.Remove(_sAddedSQLFilter.Length - 5);
                    if (_sSQLQuery.EndsWith(",") || _sSQLQuery.EndsWith(";"))
                        _sSQLQuery = _sSQLQuery.Remove(_sSQLQuery.Length - 1);

                    
                    if (_sAddedSQLFilter.Length > 1 && _sAddedSQLQuery.Length > 1 && _sOrderByQuery.Length > 1)
                        _report.DataSource.RecordSource = "PARAMETERS " + _sAddedSQLQuery + _sSQLQuery + " WHERE " + _sAddedSQLFilter + " ORDER BY " + _sOrderByQuery + ";";
                    else if (_sAddedSQLFilter.Length > 1 && _sAddedSQLQuery.Length > 1 && _sOrderByQuery.Length <= 0)
                        _report.DataSource.RecordSource = "PARAMETERS " + _sAddedSQLQuery + _sSQLQuery + " WHERE " + _sAddedSQLFilter + ";";
                    else if (_sAddedSQLFilter.Length > 1 && _sAddedSQLQuery.Length <= 0 && _sOrderByQuery.Length > 1)
                        _report.DataSource.RecordSource = _sSQLQuery + " WHERE " + _sAddedSQLFilter + " ORDER BY " + _sOrderByQuery + ";";
                    else if (_sAddedSQLFilter.Length > 1 && _sAddedSQLQuery.Length <= 0 && _sOrderByQuery.Length <= 0)
                        _report.DataSource.RecordSource = _sSQLQuery + " WHERE " + _sAddedSQLFilter + ";";
                    else if (_sAddedSQLFilter.Length <= 0 && _sAddedSQLQuery.Length > 1 && _sOrderByQuery.Length > 1)
                        _report.DataSource.RecordSource = "PARAMETERS " + _sAddedSQLQuery + _sSQLQuery + " ORDER BY " + _sOrderByQuery + ";";
                    else if (_sAddedSQLFilter.Length <= 0 && _sAddedSQLQuery.Length > 1 && _sOrderByQuery.Length <= 0)
                        _report.DataSource.RecordSource = "PARAMETERS " + _sAddedSQLQuery + _sSQLQuery + ";";
                    else if (_sAddedSQLFilter.Length <= 0 && _sAddedSQLQuery.Length <= 0 && _sOrderByQuery.Length > 1)
                        _report.DataSource.RecordSource = _sSQLQuery + " ORDER BY " + _sOrderByQuery + ";";
                    else
                        _report.DataSource.RecordSource = _sSQLQuery + ";";
                //}
                //else
                //{
                //    if(_sOrderByQuery.Length >1)
                //        _report.DataSource.RecordSource = _sSQLQuery + " ORDER BY " + _sOrderByQuery + ";";
                //    else
                //        _report.DataSource.RecordSource = _sSQLQuery + ";";
                //}
                MessageBox.Show("SQL Query Modified Successfully");
                this.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void _btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void _bAddFilter_Click(object sender, EventArgs e)
        {
            try
            {
                if (_lbFieldList.SelectedItems.Count > 0 && _cbOperatorType.SelectedIndex != -1 && _lbParamsList.SelectedItems.Count > 0)
                {
                    if (_cbOperatorType.SelectedIndex != 6)
                    {
                        if (_lbParamsList.SelectedItems.Count == 1)
                            _lbFiltersList.Items.Add(_lbFieldList.SelectedItem.ToString() + " " + _cbOperatorType.SelectedItem.ToString() + " [" + _lbParamsList.SelectedItem.ToString() + "]");
                        else
                            MessageBox.Show("Select Only one Parameters from Parameter List");
                    }
                    else
                    {
                        if (_lbParamsList.SelectedItems.Count == 2)
                        {
                            _lbFiltersList.Items.Add(_lbFieldList.SelectedItem.ToString() + " " + _cbOperatorType.SelectedItem.ToString() + " [" + _lbParamsList.SelectedItems[0].ToString() + "] AND " + "[" + _lbParamsList.SelectedItems[1].ToString() + "]");
                        }
                        else
                        {
                            MessageBox.Show("Select two Parameters from Parameters List");
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Select at least one entry from each of Field, Operator and Parameters");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void _btnRemoveSelectedFilter_Click(object sender, EventArgs e)
        {
            if (_lbFiltersList.SelectedIndex != -1)
            {
                _lbFiltersList.Items.RemoveAt(_lbFiltersList.SelectedIndex);
            }
        }

        private void _btnRemoveAllFilters_Click(object sender, EventArgs e)
        {
            _lbFiltersList.Items.Clear();
        }

    }
}
