//----------------------------------------------------------------------------
// DataSourceForm.cs
//
// DataSource picker form for use with C1ReportDesigner app
// This is just a dialog that contains the DataSourcePicker control.
//
// This is a part of the C1ReportDesigner package.
// Copyright (C) GrapeCity, Inc
// All rights reserved.
//
// This source code is intended as a supplement to the C1ReportDesigner 
// control and related documentation provided with the product.
//
// See these sources for detailed information regarding the
// C1ReportDesigner product.
//----------------------------------------------------------------------------
using System;
using System.Data;
using System.Data.Common;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

using C1.C1Report;

namespace ReportDesignerApp.Util
{
	/// <summary>
    /// DataSource picker form for use with C1ReportDesigner app.
	/// </summary>
	public class DataSourceForm : System.Windows.Forms.Form
    {
        // private fields
        private DataSourcePicker _dspMain;
        private Button _btnOK;
        private Button _btnCancel;
        private C1Report _report;
        private Panel panel1;

        /// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public DataSourceForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
            
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(DataSourceForm));
            this._dspMain = new ReportDesignerApp.Util.DataSourcePicker();
            this._btnOK = new System.Windows.Forms.Button();
            this._btnCancel = new System.Windows.Forms.Button();
            this.panel1 = new System.Windows.Forms.Panel();
            this.panel1.SuspendLayout();
            this.SuspendLayout();
            // 
            // _dspMain
            // 
            this._dspMain.Dock = System.Windows.Forms.DockStyle.Fill;
            this._dspMain.Location = new System.Drawing.Point(0, 0);
            this._dspMain.Name = "_dspMain";
            this._dspMain.Size = new System.Drawing.Size(472, 308);
            this._dspMain.TabIndex = 22;
            // 
            // _btnOK
            // 
            this._btnOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this._btnOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
            this._btnOK.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this._btnOK.Location = new System.Drawing.Point(313, 8);
            this._btnOK.Name = "_btnOK";
            this._btnOK.Size = new System.Drawing.Size(75, 23);
            this._btnOK.TabIndex = 23;
            this._btnOK.Text = "&OK";
            this._btnOK.Click += new System.EventHandler(this._btnOK_Click);
            // 
            // _btnCancel
            // 
            this._btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this._btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this._btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
            this._btnCancel.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this._btnCancel.Location = new System.Drawing.Point(394, 8);
            this._btnCancel.Name = "_btnCancel";
            this._btnCancel.Size = new System.Drawing.Size(75, 23);
            this._btnCancel.TabIndex = 24;
            this._btnCancel.Text = "&Cancel";
            // 
            // panel1
            // 
            this.panel1.Controls.Add(this._btnCancel);
            this.panel1.Controls.Add(this._btnOK);
            this.panel1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.panel1.Location = new System.Drawing.Point(0, 308);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(472, 38);
            this.panel1.TabIndex = 25;
            // 
            // DataSourceForm
            // 
            this.AcceptButton = this._btnOK;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.CancelButton = this._btnCancel;
            this.ClientSize = new System.Drawing.Size(472, 346);
            this.Controls.Add(this._dspMain);
            this.Controls.Add(this.panel1);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.MinimumSize = new System.Drawing.Size(460, 300);
            this.Name = "DataSourceForm";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
            this.Text = "Select a Data Source";
            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.DataSourceForm_FormClosing);
            this.Load += new System.EventHandler(this.DataSourceForm_Load);
            this.panel1.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		#endregion

        #region Public
        // bind dialog to a report and show it
        public bool EditDataSource(
            C1Report report,
            Hashtable dataSchema)
        {
            // save reference to report being edited
            _report = report;

            // initialize picker control
            _dspMain.DataSchema = dataSchema;

            // initialize members
            DataSource ds = _report.DataSource;
            _dspMain.IsolationLevel = _report.DataSource.IsolationLevel;
            _dspMain.SetDataSource(_report.DataSource.DataProvider, _report.DataSource.ConnectionString, _report.DataSource.RecordSource);

            // set up form events
            // _dspMain.DoubleClick += new EventHandler(_btnOK_Click);

            // ready to go
            return ShowDialog() == DialogResult.OK;
        }
        #endregion

        #region Nested types
        private class DataSourceFormState : FormState
        {
            #region Constructors
            public DataSourceFormState(DataSourceForm owner)
                : base(owner)
            {
            }
            #endregion

            #region Internal properties
            internal new DataSourceForm Owner
            {
                get { return (DataSourceForm)base.Owner; }
            }
            #endregion

            #region Public properties
            [DefaultValue(false)]
            public bool Wrap
            {
                get { return Owner._dspMain.WordWrap; }
                set { Owner._dspMain.WordWrap = value; }
            }
            #endregion
        }
        #endregion

        // save results and dismiss dialog
		private void _btnOK_Click(object sender, System.EventArgs e)
		{
            if (!_dspMain.RecordSourceSelected)
            {
                // AppUtils.ShowWarning(Strings.DataSourceForm.SelectDataSource);
                MessageForm.Warn(Strings.DataSourceForm.SelectDataSource);
                return;
            }

			// apply changes
            DataSource ds = _report.DataSource;
            ds.DataProvider = _dspMain.DataProvider;
            ds.ConnectionString = _dspMain.ConnectionString;
            ds.RecordSource = _dspMain.RecordSource;
            ds.Recordset = _dspMain.Recordset;

			// done
			DialogResult = DialogResult.OK;
		}

        private void DataSourceForm_Load(object sender, EventArgs e)
        {
            AppSettings.LoadObject("DataSourceForm", new DataSourceFormState(this));
        }

        private void DataSourceForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            AppSettings.SaveObject("DataSourceForm", new DataSourceFormState(this));
        }
	}
}
