//----------------------------------------------------------------------------
// Utils.cs
//
// This is a part of the C1ReportDesigner package.
// Copyright (C) GrapeCity, Inc
// All rights reserved.
//
// This source code is intended as a supplement to the C1ReportDesigner 
// control and related documentation provided with the product.
//
// See these sources for detailed information regarding the
// C1ReportDesigner product.
//----------------------------------------------------------------------------
using System;
using System.IO;
using System.Resources;
using System.Drawing;
using System.Drawing.Imaging;
using System.Reflection;
using System.Windows.Forms;

namespace ReportDesignerApp.Util
{
    internal static class AppUtils
    {
        #region Public static
        public static Image LoadResourceImage(
            Assembly assembly,
            string resourceName)
        {
            foreach (string res in assembly.GetManifestResourceNames())
            {
                if (res.EndsWith(resourceName))
                {
                    System.IO.Stream stream = assembly.GetManifestResourceStream(res);
                    return Image.FromStream(stream);
                }
            }
            return null;
        }

        public static Image LoadResourceImage(
            string resourceName)
        {
            return LoadResourceImage(Assembly.GetExecutingAssembly(), resourceName);
        }

        public static void ParseVersionString(
            string version,
            out int major,
            out int minor,
            out int build,
            out int rev)
        {
            major = minor = build = rev = 0;
            if (!string.IsNullOrEmpty(version))
            {
                var vs = version.Split(new char[] { '.' });
                if (vs.Length > 0)
                    int.TryParse(vs[0], out major);
                if (vs.Length > 1)
                    int.TryParse(vs[1], out minor);
                if (vs.Length > 2)
                    int.TryParse(vs[2], out build);
                if (vs.Length > 3)
                    int.TryParse(vs[3], out rev);
            }
        }

        public static TreeNode FindNodeByTag(
            TreeNodeCollection nodes,
            object tagValue)
        {
            foreach (TreeNode tn in nodes)
            {
                if (tn.Tag == tagValue)
                    return tn;
                TreeNode result = FindNodeByTag(tn.Nodes, tagValue);
                if (result != null)
                    return result;
            }
            return null;
        }

        public static TreeNode FindNodeByText(
            TreeNodeCollection nodes,
            string text,
            bool ignoreCase)
        {
            foreach (TreeNode tn in nodes)
            {
                if (string.Compare(tn.Text, text, ignoreCase) == 0)
                    return tn;
                TreeNode result = FindNodeByText(tn.Nodes, text, ignoreCase);
                if (result != null)
                    return result;
            }
            return null;
        }
        #endregion
    }
}
